/* 
 * Copyright (C) 2006 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#ifndef BITTWIDDLER_H
#define BITTWIDDLER_H

#include "common.h"

/* accumulates bits until we have a byte */
typedef struct {
  byte_t b;
  unsigned char p;
} bittwiddler_t;

/*  we don't want the literal constant 8 all over the place ;) */
#define BITSPERBYTE 8

/* inlined functions. no error or consistency checking */

static __inline__
void setbyte_bittwiddler(bittwiddler_t *t, byte_t b) {
  t->b = b;
  t->p = 0;
}

static __inline__
bool_t flush_bittwiddler(bittwiddler_t *t) {
  if( t->p < BITSPERBYTE ) {
    t->b = (t->b<<(BITSPERBYTE - t->p));
    t->p = BITSPERBYTE;
    return TRUE;
  }
  return FALSE;
}

static __inline__
byte_t getbyte_bittwiddler(bittwiddler_t *t) {
  flush_bittwiddler(t);
  return t->b;
}

static __inline__
bool_t numbits_bittwiddler(bittwiddler_t *t) {
  return t->p;
}

static __inline__
bool_t isfull_bittwiddler(bittwiddler_t *t) {
  return (t->p >= BITSPERBYTE);
}

static __inline__
bool_t isempty_bittwiddler(bittwiddler_t *t) {
  return (t->p == 0) && (t->b == 0);
}

static __inline__
bool_t appendbit_bittwiddler(bittwiddler_t *t, unsigned int v) {
  if( t->p < BITSPERBYTE ) {
    t->b = (t->b<<1) + v;
    t->p++;
    return TRUE;
  }
  return FALSE;
}

static __inline__
bool_t rewind_bittwiddler(bittwiddler_t *t) { 
  t->p = 0;
  return TRUE;
}

static __inline__
bool_t readnextbit_bittwiddler(bittwiddler_t *t, unsigned int *v) {
  *v = ((t->b & (1<<(BITSPERBYTE - 1 - t->p))) > 0); 
  t->p++;
  return (t->p <= BITSPERBYTE);
}

static __inline__
int bits2bytes(unsigned int n) {
  return (n / BITSPERBYTE) + ((n % BITSPERBYTE > 0) ? 1 : 0);
}

#endif
