/* 
 * Copyright (C) 2008 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#include "ppmwriter.h"
#include "util.h"
#include "bittwiddler.h"

#include <unistd.h>
#include <stdlib.h>
#include <string.h>

extern size_t path_max;

bool_t create_ppmwriter(ppmwriter_t *p, int fd, imagetype_t typ) {
  if( p ) {
    p->type = typ;
    p->colors = 0;
    p->fd = fd;
    p->f = fdopen(p->fd, "wb");
    if( p->f ) {
      return TRUE;
    }
  }
  return FALSE;
}

bool_t free_ppmwriter(ppmwriter_t *p) {
  if( p ) {
    if( p->f ) {
      fclose(p->f);
      p->f = NULL;
      p->fd = -1;
    }
  }
  return FALSE;
}


bool_t write_header_ppmwriter(ppmwriter_t *p, 
			      unsigned int width, unsigned int height,
			      unsigned int colors, imagetype_t typ) {
  int n;
  if( p && p->f ) {
    p->type = typ;
    p->colors = colors;
    switch(p->type) {
    case tP6:
      n = fprintf(p->f, "P6\n%d %d\n%d\n", width, height, colors);
      break;
    default:
      n = -1;
    }
    if( n < 0 ) {
      /* major problem, should never happen */
      free_ppmwriter(p);
      return FALSE;
    }
    /* this is mainly for consistency checks later */ 
    p->imagesize = width * height;
    p->byteswritten = 0;
    return TRUE;
  }
  return FALSE;
}

bool_t write_pixels_ppmwriter(ppmwriter_t *p, 
			      byte_t *buf, unsigned int buflen) {
  int n, j;
  if( p && p->f ) {
    for(j = 0; j < buflen; ) {
      n = fwrite(buf + 3 * j, 3, buflen - j, p->f);
      if( n <= 0 ) {
	return FALSE;
      }
      j += n;
    }
    p->byteswritten += 3 * buflen;
    return TRUE;
  }
  return FALSE;
}


bool_t consistency_check_ppmwriter(ppmwriter_t *p) {
  switch(p->type) {
  case tP6:
    return ( p && p->f && (p->byteswritten * BITSPERBYTE >= p->imagesize) );
  default:
    break;
  }
  return FALSE;
}

